@@ -7,8 +7,24 @@
 #include <zlib.h>
 #include <png.h>
 #include <stdlib.h>
+#ifdef _MSC_VER
+#define _USE_MATH_DEFINES 1
+#include <math.h>
+
+float rint (float x)
+{
+     return floor (x + 0.5f);
+}
+
+float trunc (float x)
+{
+  return (((x) < 0) ? ceil ((x)) : floor ((x)));
+} 
+#define unlink _unlink
+#else
 #include <unistd.h>
 #include <math.h>
+#endif
 
 // A support of 3 gives an overall sharper looking image, but
 // it is a) slower b) gives more sharpening artefacts
@@ -94,7 +110,7 @@ bool Image::LoadPNG ()
 
     png_init_io (png, file);
 
-    if (setjmp (png->jmpbuf))
+    if (setjmp (png_jmpbuf(png)))
         // If we get here, we had a problem reading the file
         goto nomem;
 
@@ -157,7 +173,7 @@ bool Image::LoadPNG ()
     row_pointers = new png_bytep [Height];
 
     if (!row_pointers
-        || setjmp (png->jmpbuf))             // Set a new exception handler
+        || setjmp (png_jmpbuf(png)))             // Set a new exception handler
     {
         delete [] row_pointers;
     nomem:
@@ -214,7 +230,7 @@ bool Image::SavePNG (const char *fName)
     }
 
     /* Catch processing errors */
-    if (setjmp(png->jmpbuf))
+    if (setjmp(png_jmpbuf(png)))
         /* If we get here, we had a problem writing the file */
         goto error2;
 
@@ -273,10 +289,6 @@ bool Image::SavePNG (const char *fName)
     /* It is REQUIRED to call this to finish writing the rest of the file */
     png_write_end (png, info);
 
-    /* if you malloced the palette, free it here */
-    if (info->palette)
-        free (info->palette);
-
     /* clean up after the write, and free any memory allocated */
     png_destroy_write_struct (&png, &info);
 
