@@ -11,6 +11,7 @@
 #include <regex.h>
 #include <locale.h>
 #include <math.h>
+#include <algorithm>
 
 static struct
 {
@@ -34,7 +35,7 @@ static struct
     },
     {
         // [min focal]-[max focal]mm f/[min aperture]-[max aperture]
-        "([0-9.]+)(-[0-9.]+)?(mm)?[[:space:]]+(f/?)?([0-9.]+)(-[0-9.]+)?",
+        "([0-9]+[0-9.]*)(-[0-9]+[0-9.]*)?(mm)?[[:space:]]+(f/?)?([0-9.]+)(-[0-9.]+)?",
         { 1, 2, 5, 6 },
         false
     }
@@ -123,6 +124,8 @@ lfLens::~lfLens ()
     _lf_list_free ((void **)CalibDistortion);
     _lf_list_free ((void **)CalibTCA);
     _lf_list_free ((void **)CalibVignetting);
+    _lf_list_free ((void **)CalibCrop);
+    _lf_list_free ((void **)CalibFov);
     if (!--_lf_lens_regex_refs)
         _lf_free_lens_regex ();
 }
@@ -163,6 +166,14 @@ lfLens &lfLens::operator = (const lfLens &other)
     if (other.CalibVignetting)
         for (int i = 0; other.CalibVignetting [i]; i++)
             AddCalibVignetting (other.CalibVignetting [i]);
+    lf_free (CalibCrop); CalibCrop = NULL;
+    if (other.CalibCrop)
+        for (int i = 0; other.CalibCrop [i]; i++)
+            AddCalibCrop (other.CalibCrop [i]);
+    lf_free (CalibFov); CalibFov = NULL;
+    if (other.CalibFov)
+        for (int i = 0; other.CalibFov [i]; i++)
+            AddCalibFov (other.CalibFov [i]);
 
     return *this;
 }
@@ -188,14 +199,16 @@ void lfLens::GuessParameters ()
     float minf = float (INT_MAX), maxf = float (INT_MIN);
     float mina = float (INT_MAX), maxa = float (INT_MIN);
 
-    char *old_numeric = setlocale (LC_NUMERIC, "C");
+    char *old_numeric = setlocale (LC_NUMERIC, NULL);
+    old_numeric = strdup (old_numeric);
+    setlocale (LC_NUMERIC,"C");
 
     if (!MinAperture || !MinFocal)
         _lf_parse_lens_name (Model, minf, maxf, mina, maxa);
 
     if (!MinAperture || !MinFocal)
     {
-        // Try to find out the range of focal distance using calibration data
+        // Try to find out the range of focal lengths using calibration data
         if (CalibDistortion)
             for (int i = 0; CalibDistortion [i]; i++)
             {
@@ -228,6 +241,25 @@ void lfLens::GuessParameters ()
                 if (a > maxa)
                     maxa = a;
             }
+        if (CalibCrop)
+            for (int i=0; CalibCrop [i]; i++)
+            {
+                float f = CalibCrop [i]->Focal;
+                if (f < minf)
+                    minf = f;
+                if (f > maxf)
+                    maxf = f;
+            }
+        if (CalibFov)
+            for (int i=0; CalibFov [i]; i++)
+            {
+                float f = CalibFov [i]->Focal;
+                if (f < minf)
+                    minf = f;
+                if (f > maxf)
+                    maxf = f;
+            }
+
     }
 
     if (minf != INT_MAX && !MinFocal)
@@ -245,6 +277,7 @@ void lfLens::GuessParameters ()
         MaxAperture = MinAperture;
 
     setlocale (LC_NUMERIC, old_numeric);
+    free (old_numeric);
 }
 
 bool lfLens::Check ()
@@ -289,7 +322,7 @@ const char *lfLens::GetDistortionModelDesc (
 
         case LF_DIST_MODEL_POLY3:
             if (details)
-                *details = "Ru = Rd * (1 + k1 * Rd^2)\n"
+                *details = "Ru = Rd * (1 - k1 + k1 * Rd^2)\n"
                     "Ref: http://www.imatest.com/docs/distortion.html";
             if (params)
                 *params = param_poly3;
@@ -313,7 +346,7 @@ const char *lfLens::GetDistortionModelDesc (
 
         case LF_DIST_MODEL_PTLENS:
             if (details)
-                *details = "Ru = Rd * (a * Rd^3 + b * Rd^2 + c * Rd + 1)\n"
+                *details = "Ru = Rd * (a * Rd^3 + b * Rd^2 + c * Rd + 1 - (a + b + c))\n"
                     "Ref: http://wiki.panotools.org/Lens_correction_model";
             if (params)
                 *params = param_ptlens;
@@ -416,7 +449,7 @@ const char *lfLens::GetVignettingModelDesc (
             if (details)
                 *details = "Pablo D'Angelo vignetting model\n"
                     "(which is a more general variant of the cos^4 law):\n"
-                    "c = 1 + k1 * R^2 + k2 * R^4 + k3 * R^6\n"
+                    "Cd = Cs * (1 + k1 * R^2 + k2 * R^4 + k3 * R^6)\n"
                     "Ref: http://hugin.sourceforge.net/tech/";
             if (params)
                 *params = param_pa;
@@ -434,6 +467,52 @@ const char *lfLens::GetVignettingModelDesc (
     return NULL;
 }
 
+const char *lfLens::GetCropDesc (
+    lfCropMode mode , const char **details, const lfParameter ***params)
+{
+    static const lfParameter *param_none [] = { NULL };
+
+    static const lfParameter param_crop_left = { "left", -1.0F, 1.0F, 0.0F };
+    static const lfParameter param_crop_right = { "right", 0.0F, 2.0F, 0.0F };
+    static const lfParameter param_crop_top = { "top", -1.0F, 1.0F, 0.0F };
+    static const lfParameter param_crop_bottom = { "bottom", 0.0F, 2.0F, 0.0F };
+    static const lfParameter *param_crop [] = { &param_crop_left, &param_crop_right, &param_crop_top, &param_crop_bottom, NULL };
+
+    switch (mode)
+    {
+        case LF_NO_CROP:
+            if (details)
+                *details = "No crop";
+            if (params)
+                *params = param_none;
+            return "No crop";
+
+        case LF_CROP_RECTANGLE:
+            if (details)
+                *details = "Rectangular crop area";
+            if (params)
+                *params = param_crop;
+            return "rectangular crop";
+
+        case LF_CROP_CIRCLE:
+            if (details)
+                *details = "Circular crop area";
+            if (params)
+                *params = param_crop;
+            return "circular crop";
+
+        default:
+            // keep gcc 4.4 happy
+            break;
+    }
+
+    if (details)
+        *details = NULL;
+    if (params)
+        *params = NULL;
+    return NULL;
+}
+
 const char *lfLens::GetLensTypeDesc (lfLensType type, const char **details)
 {
     switch (type)
@@ -463,6 +542,26 @@ const char *lfLens::GetLensTypeDesc (lfLensType type, const char **details)
                 *details = "Ref: http://wiki.panotools.org/Equirectangular_Projection";
             return "Equirectangular";
 
+        case LF_FISHEYE_ORTHOGRAPHIC:
+            if (details)
+                *details = "Ref: http://wiki.panotools.org/Fisheye_Projection";
+            return "Fisheye, orthographic";
+
+        case LF_FISHEYE_STEREOGRAPHIC:
+            if (details)
+                *details = "Ref: http://wiki.panotools.org/Stereographic_Projection";
+            return "Fisheye, stereographic";
+
+        case LF_FISHEYE_EQUISOLID:
+            if (details)
+                *details = "Ref: http://wiki.panotools.org/Fisheye_Projection";
+            return "Fisheye, equisolid";
+
+        case LF_FISHEYE_THOBY:
+            if (details)
+                *details = "Ref: http://groups.google.com/group/hugin-ptx/browse_thread/thread/bd822d178e3e239d";
+            return "Thoby-Fisheye";
+
         default:
             // keep gcc 4.4 happy
             break;
@@ -562,6 +661,64 @@ bool lfLens::RemoveCalibVignetting (int idx)
     return _lf_delobj (x.arr, idx);
 }
 
+static bool cmp_lenscrop (const void *x1, const void *x2)
+{
+    const lfLensCalibCrop *d1 = static_cast<const lfLensCalibCrop *> (x1);
+    const lfLensCalibCrop *d2 = static_cast<const lfLensCalibCrop *> (x2);
+    return (d1->Focal == d2->Focal);
+}
+
+void lfLens::AddCalibCrop (const lfLensCalibCrop *lcc)
+{
+    // Avoid "dereferencing type-punned pointer will break strict-aliasing rules" warning
+    union
+    {
+        lfLensCalibCrop ***cd;
+        void ***arr;
+    } x = { &CalibCrop };
+    _lf_addobj (x.arr, lcc, sizeof (*lcc), cmp_lenscrop);
+}
+
+bool lfLens::RemoveCalibCrop (int idx)
+{
+    // Avoid "dereferencing type-punned pointer will break strict-aliasing rules" warning
+    union
+    {
+        lfLensCalibCrop ***cd;
+        void ***arr;
+    } x = { &CalibCrop };
+    return _lf_delobj (x.arr, idx);
+}
+
+static bool cmp_lensfov (const void *x1, const void *x2)
+{
+    const lfLensCalibFov *d1 = static_cast<const lfLensCalibFov *> (x1);
+    const lfLensCalibFov *d2 = static_cast<const lfLensCalibFov *> (x2);
+    return (d1->Focal == d2->Focal);
+}
+
+void lfLens::AddCalibFov (const lfLensCalibFov *lcf)
+{
+    // Avoid "dereferencing type-punned pointer will break strict-aliasing rules" warning
+    union
+    {
+        lfLensCalibFov ***cd;
+        void ***arr;
+    } x = { &CalibFov };
+    _lf_addobj (x.arr, lcf, sizeof (*lcf), cmp_lensfov);
+}
+
+bool lfLens::RemoveCalibFov (int idx)
+{
+    // Avoid "dereferencing type-punned pointer will break strict-aliasing rules" warning
+    union
+    {
+        lfLensCalibFov ***cd;
+        void ***arr;
+    } x = { &CalibFov };
+    return _lf_delobj (x.arr, idx);
+}
+
 static int __insert_spline (void **spline, float *spline_dist, float dist, void *val)
 {
     if (dist < 0)
@@ -738,7 +895,7 @@ bool lfLens::InterpolateTCA (float focal, lfLensCalibTCA &res) const
 }
 
 static float __vignetting_dist (
-    const lfLens *l, lfLensCalibVignetting &x, float focal, float aperture, float distance)
+    const lfLens *l, const lfLensCalibVignetting &x, float focal, float aperture, float distance)
 {
     // translate every value to linear scale and normalize
     // approximatively to range 0..1
@@ -750,81 +907,40 @@ static float __vignetting_dist (
         f1 /= df;
         f2 /= df;
     }
-    float a1 = log (aperture) / 2.77258872223978123766;   // log(16)
-    float a2 = log (x.Aperture) / 2.77258872223978123766; // log(16)
-    float d1 = log (distance);
-    float d2 = log (x.Distance);
+    float a1 = 8.0 / aperture;
+    float a2 = 8.0 / x.Aperture;
+    float d1 = 0.1 / distance;
+    float d2 = 0.1 / x.Distance;
 
     return sqrt (square (f2 - f1) + square (a2 - a1) + square (d2 - d1));
 }
 
-static void __vignetting_interp (
-    lfLensCalibVignetting **spline, lfLensCalibVignetting &res, float t)
-{
-    res.Focal = _lf_interpolate (
-        spline [0] ? spline [0]->Focal : FLT_MAX,
-        spline [1]->Focal, spline [2]->Focal,
-        spline [3] ? spline [3]->Focal : FLT_MAX, t);
-    res.Aperture = _lf_interpolate (
-        spline [0] ? spline [0]->Aperture : FLT_MAX,
-        spline [1]->Aperture, spline [2]->Aperture,
-        spline [3] ? spline [3]->Aperture : FLT_MAX, t);
-    res.Distance = _lf_interpolate (
-        spline [0] ? spline [0]->Distance : FLT_MAX,
-        spline [1]->Distance, spline [2]->Distance,
-        spline [3] ? spline [3]->Distance : FLT_MAX, t);
-}
-
 bool lfLens::InterpolateVignetting (
     float focal, float aperture, float distance, lfLensCalibVignetting &res) const
 {
     if (!CalibVignetting)
         return false;
 
-    /* This is a pretty complex problem, despite its apparent simplicity.
-     * What we have is to find the intensity of a 3D force field in given
-     * point (let's call it p) determined by coordinates (X=focal,
-     * Y=aperture, Z=distance). For this we have a lot of measurements
-     * (e.g. calibration data) taken at other random points with their
-     * own X/Y/Z coordinates.
-     *
-     * We must find the line that intersects at least two calibration
-     * points and comes as close to the interpolation point as possible.
-     * Even better if we find 4 such points, in this case we can approximate
-     * by a real spline instead of doing linear interpolation.
-     *
-     * We will do this as follows. For every point find the other points
-     * on the line that intersects this point and the point p. We need
-     * just four points: two on one side of p, and two on other side of it.
-     * Then build a Hermit spline that passes through these four points.
-     * Finally, compute a estimate of the "quality" of this spline, which
-     * is made of: total spline length (the shorter is spline, the higher
-     * is quality), how close the resulting X,Y,Z is to our target.
-     * When we get a estimate of a high enough quality, we return
-     * the found point to the caller.
-     *
-     * Otherwise, the found point is added to the pool and the loop goes
-     * over and over again.
-     */
-
-    bool rc = false;
-
-    GPtrArray *vc = g_ptr_array_new ();
-    int cvc;
-    for (cvc = 0; CalibVignetting [cvc]; cvc++)
-        g_ptr_array_add (vc, CalibVignetting [cvc]);
-
     lfVignettingModel vm = LF_VIGNETTING_MODEL_NONE;
+    res.Focal = focal;
+    res.Aperture = aperture;
+    res.Distance = distance;
+    for (size_t i = 0; i < ARRAY_LEN (res.Terms); i++)
+        res.Terms [i] = 0;
 
-    float min_dist = 0.01F;
-    for (guint i = 0; i < vc->len; i++)
+    float total_weighting = 0;
+    const float power = 3.5;
+
+    float smallest_interpolation_distance = FLT_MAX;
+    for (int i = 0; CalibVignetting [i]; i++)
     {
-        lfLensCalibVignetting *c =
-            (lfLensCalibVignetting *)g_ptr_array_index (vc, i);
-
+        const lfLensCalibVignetting* c = CalibVignetting [i];
         // Take into account just the first encountered lens model
         if (vm == LF_VIGNETTING_MODEL_NONE)
+	    {
             vm = c->Model;
+	        res.Model = vm;
+        } 
         else if (vm != c->Model)
         {
             g_warning ("WARNING: lens %s/%s has multiple vignetting models defined\n",
@@ -832,188 +948,158 @@ bool lfLens::InterpolateVignetting (
             continue;
         }
 
-        //                    __
-        // Compute the vector pc
-        float pcx = c->Focal - focal;
-        float pcy = c->Aperture - aperture;
-        float pcz = c->Distance - distance;
-        float norm = sqrt (pcx * pcx + pcy * pcy + pcz * pcz);
+	    float interpolation_distance = __vignetting_dist (this, *c, focal, aperture, distance);
+	    if (interpolation_distance < 0.0001) {
+	        res = *c;
+	        return true;
+	    }
+	    
+	    smallest_interpolation_distance = std::min(smallest_interpolation_distance, interpolation_distance);
+	    float weighting = abs (1.0 / pow (interpolation_distance, power));
+	    for (size_t i = 0; i < ARRAY_LEN (res.Terms); i++)
+	        res.Terms [i] += weighting * c->Terms [i];
+	    total_weighting += weighting;
+    }
+    
+    if (smallest_interpolation_distance > 1)
+        return false;
+    
+    if (total_weighting > 0 && smallest_interpolation_distance < FLT_MAX)
+    {
+	    for (size_t i = 0; i < ARRAY_LEN (res.Terms); i++)
+	        res.Terms [i] /= total_weighting;
+	    return true;
+    } else 
+        return false;
+}
 
-        // If c == p, return it without any interpolation
-        if (norm < 0.0001)
-        {
-            res = *c;
-            rc = true;
-            goto leave;
-        }
+bool lfLens::InterpolateCrop (float focal, lfLensCalibCrop &res) const
+{
+    if (!CalibCrop)
+        return false;
 
-        norm = 1.0 / norm;
-        pcx *= norm;
-        pcy *= norm;
-        pcz *= norm;
+    union
+    {
+        lfLensCalibCrop *spline [4];
+        void *spline_ptr [4];
+    };
+    float spline_dist [4] = { -FLT_MAX, -FLT_MAX, FLT_MAX, FLT_MAX };
+    lfCropMode cm = LF_NO_CROP;
 
-        union
-        {
-            lfLensCalibVignetting *spline [4];
-            void *spline_ptr [4];
-        };
-        // Don't pick up way off points
-        float spline_rating [4] = { -10.0, -10.0, 1.0, 10.0 };
-        float spline_dist [4] =  { FLT_MAX, FLT_MAX, 1.0, FLT_MAX };
-
-        memset (&spline, 0, sizeof (spline));
-        for (guint j = 0; j < vc->len; j++)
-        {
-            if (j == i)
-                continue;
-
-            lfLensCalibVignetting *x =
-                (lfLensCalibVignetting *)g_ptr_array_index (vc, j);
-
-            //                      __
-            // Calculate the vector px and bring it to same scale
-            float pxx = x->Focal - focal;
-            float pxy = x->Aperture - aperture;
-            float pxz = x->Distance - distance;
-            float norm2 = 1.0 / sqrt (pxx * pxx + pxy * pxy + pxz * pxz);
-
-            //                                                      __     __
-            // Compute the cosinus of the angle between the vectors px and pc
-            float cs = (pxx * pcx + pxy * pcy + pxz * pcz) * norm2;
-            if (cs > -0.01 && cs < +0.01)
-                continue; // +-90 degrees
-
-            // We will judge how good this point is for us by computing
-            // the rating as the relation distance/cos(angle)^3
-            float dist = sqrt (square (pxx * norm) + square (pxy * norm) + square (pxz * norm));
-            float rating = dist / (cs * cs * cs);
-
-            if (rating >= -0.00001 && dist <= +0.00001)
-            {
-                res = *x;
-                rc = true;
-                goto leave;
-            }
-
-            switch (__insert_spline (spline_ptr, spline_rating, rating, x))
-            {
-                case 0:
-                    spline_dist [0] = dist;
-                    break;
-
-                case 1:
-                    spline_dist [0] = spline_dist [1];
-                    spline_dist [1] = dist;
-                    break;
-
-                case 2:
-                    spline_dist [3] = spline_dist [2];
-                    spline_dist [2] = dist;
-                    break;
-
-                case 3:
-                    spline_dist [3] = dist;
-                    break;
-            }
-        }
-
-        // If we have found no points for the spline, drop
-        if (!spline [1] || !spline [2])
+    memset (spline, 0, sizeof (spline));
+    for (int i = 0; CalibCrop [i]; i++)
+    {
+        lfLensCalibCrop *c = CalibCrop [i];
+        if (c->CropMode == LF_NO_CROP)
             continue;
 
-        // Sort the spline points according to the real distance
-        // between p and the points, not by "rating".
-        if (spline_dist [0] < spline_dist [1])
+        // Take into account just the first encountered crop mode
+        if (cm == LF_NO_CROP)
+            cm = c->CropMode;
+        else if (cm != c->CropMode)
         {
-            lfLensCalibVignetting *tmp = spline [0];
-            spline [0] = spline [1];
-            spline [1] = tmp;
-            float tmpf = spline_dist [0];
-            spline_dist [0] = spline_dist [1];
-            spline_dist [1] = tmpf;
-        }
-        if (spline_dist [3] < spline_dist [2])
-        {
-            lfLensCalibVignetting *tmp = spline [2];
-            spline [2] = spline [3];
-            spline [3] = tmp;
-            float tmpf = spline_dist [2];
-            spline_dist [2] = spline_dist [3];
-            spline_dist [3] = tmpf;
+            g_warning ("WARNING: lens %s/%s has multiple crop modes defined\n",
+                       Maker, Model);
+            continue;
         }
 
-        // Interpolate a new point given four spline points
-        // For this we have to find first the 't' parameter
-        // in the range 0..1 which gives the closest to p point
-        lfLensCalibVignetting m;
-        float t = 0.5;
-        float w = 0.25;
-
-        while (w > 0.001)
+        float df = focal - c->Focal;
+        if (df == 0.0)
         {
-            __vignetting_interp (spline, m, t);
-            float md = __vignetting_dist (this, m, focal, aperture, distance);
-            if (md < 0.01)
-                break;
-
-            lfLensCalibVignetting m1;
-            __vignetting_interp (spline, m1, t + 0.01);
-            float md1 = __vignetting_dist (this, m1, focal, aperture, distance);
-
-            if (md1 > md)
-                t -= w;
-            else
-                t += w;
-            w /= 2;
+            // Exact match found, don't care to interpolate
+            res = *c;
+            return true;
         }
 
-        for (size_t i = 0; i < ARRAY_LEN (res.Terms); i++)
-            m.Terms [i] = _lf_interpolate (
-                spline [0] ? spline [0]->Terms [i] : FLT_MAX,
-                spline [1]->Terms [i], spline [2]->Terms [i],
-                spline [3] ? spline [3]->Terms [i] : FLT_MAX, t);
-
-        m.Model = vm;
-        m.Focal = _lf_interpolate (
-            spline [0] ? spline [0]->Focal : FLT_MAX,
-            spline [1]->Focal, spline [2]->Focal,
-            spline [3] ? spline [3]->Focal : FLT_MAX, t);
-        m.Aperture = _lf_interpolate (
-            spline [0] ? spline [0]->Aperture : FLT_MAX,
-            spline [1]->Aperture, spline [2]->Aperture,
-            spline [3] ? spline [3]->Aperture : FLT_MAX, t);
-        m.Distance = _lf_interpolate (
-            spline [0] ? spline [0]->Distance : FLT_MAX,
-            spline [1]->Distance, spline [2]->Distance,
-            spline [3] ? spline [3]->Distance : FLT_MAX, t);
-
-        // If interpolated point is close enough, take it
-        float dist = __vignetting_dist (this, m, focal, aperture, distance);
-        // 0.005 is a carefully manually crafted value ;-D
-        if (dist < 0.005)
-        {
-            res = m;
-            rc = true;
-            goto leave;
-        }
-        if (dist < min_dist)
-        {
-            res = m;
-            rc = true;
-        }
-
-        g_ptr_array_add (vc, new lfLensCalibVignetting (m));
+        __insert_spline (spline_ptr, spline_dist, df, c);
     }
 
-leave:
-    for (guint i = cvc; i < vc->len; i++)
+    if (!spline [1] || !spline [2])
     {
-        lfLensCalibVignetting *c =
-            (lfLensCalibVignetting *)g_ptr_array_index (vc, i);
-        delete c;
+        if (spline [1])
+            res = *spline [1];
+        else if (spline [2])
+            res = *spline [2];
+        else
+            return false;
+
+        return true;
     }
-    g_ptr_array_free (vc, TRUE);
-    return rc;
+
+    // No exact match found, interpolate the model parameters
+    res.CropMode = cm;
+    res.Focal = focal;
+
+    float t = (focal - spline [1]->Focal) / (spline [2]->Focal - spline [1]->Focal);
+
+    for (size_t i = 0; i < ARRAY_LEN (res.Crop); i++)
+        res.Crop [i] = _lf_interpolate (
+            spline [0] ? spline [0]->Crop [i] : FLT_MAX,
+            spline [1]->Crop [i], spline [2]->Crop [i],
+            spline [3] ? spline [3]->Crop [i] : FLT_MAX, t);
+
+    return true;
+}
+
+bool lfLens::InterpolateFov (float focal, lfLensCalibFov &res) const
+{
+    if (!CalibFov)
+        return false;
+
+    union
+    {
+        lfLensCalibFov *spline [4];
+        void *spline_ptr [4];
+    };
+    float spline_dist [4] = { -FLT_MAX, -FLT_MAX, FLT_MAX, FLT_MAX };
+
+    memset (spline, 0, sizeof (spline));
+    int counter=0;
+    for (int i = 0; CalibFov [i]; i++)
+    {
+        lfLensCalibFov *c = CalibFov [i];
+        if (c->FieldOfView == 0)
+            continue;
+
+        counter++;
+        float df = focal - c->Focal;
+        if (df == 0.0)
+        {
+            // Exact match found, don't care to interpolate
+            res = *c;
+            return true;
+        }
+
+        __insert_spline (spline_ptr, spline_dist, df, c);
+    }
+
+    //no valid data found
+    if (counter==0)
+        return false;
+
+    if (!spline [1] || !spline [2])
+    {
+        if (spline [1])
+            res = *spline [1];
+        else if (spline [2])
+            res = *spline [2];
+        else
+            return false;
+
+        return true;
+    }
+
+    // No exact match found, interpolate the model parameters
+    res.Focal = focal;
+
+    float t = (focal - spline [1]->Focal) / (spline [2]->Focal - spline [1]->Focal);
+
+    res.FieldOfView = _lf_interpolate (
+        spline [0] ? spline [0]->FieldOfView : FLT_MAX,
+        spline [1]->FieldOfView, spline [2]->FieldOfView,
+        spline [3] ? spline [3]->FieldOfView : FLT_MAX, t);
+
+    return true;
 }
 
 gint _lf_lens_compare (gconstpointer a, gconstpointer b)
@@ -1269,6 +1355,12 @@ const char *lf_get_vignetting_model_desc (
     return lfLens::GetVignettingModelDesc (model, details, params);
 }
 
+const char *lf_get_crop_desc (
+    enum lfCropMode mode, const char **details, const lfParameter ***params)
+{
+    return lfLens::GetCropDesc (mode, details, params);
+}
+
 const char *lf_get_lens_type_desc (enum lfLensType type, const char **details)
 {
     return lfLens::GetLensTypeDesc (type, details);
@@ -1291,6 +1383,18 @@ cbool lf_lens_interpolate_vignetting (const lfLens *lens, float focal, float ape
     return lens->InterpolateVignetting (focal, aperture, distance, *res);
 }
 
+cbool lf_lens_interpolate_crop (const lfLens *lens, float focal,
+    lfLensCalibCrop *res)
+{
+    return lens->InterpolateCrop (focal, *res);
+}
+
+cbool lf_lens_interpolate_fov (const lfLens *lens, float focal,
+    lfLensCalibFov *res)
+{
+    return lens->InterpolateFov (focal, *res);
+}
+
 void lf_lens_add_calib_distortion (lfLens *lens, const lfLensCalibDistortion *dc)
 {
     lens->AddCalibDistortion (dc);
@@ -1320,3 +1424,25 @@ cbool lf_lens_remove_calib_vignetting (lfLens *lens, int idx)
 {
     return lens->RemoveCalibVignetting (idx);
 }
+
+void lf_lens_add_calib_crop (lfLens *lens, const lfLensCalibCrop *lcc)
+{
+    lens->AddCalibCrop (lcc);
+}
+
+cbool lf_lens_remove_calib_crop (lfLens *lens, int idx)
+{
+    return lens->RemoveCalibCrop (idx);
+}
+
+
+void lf_lens_add_calib_fov (lfLens *lens, const lfLensCalibFov *lcf)
+{
+    lens->AddCalibFov (lcf);
+}
+
+cbool lf_lens_remove_calib_fov (lfLens *lens, int idx)
+{
+    return lens->RemoveCalibFov (idx);
+}
+
